@@ -5,10 +5,10 @@
 #include <qpdf/QPDFWriter.hh>
 #include <qpdf/QTC.hh>
 #include <qpdf/QUtil.hh>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <iostream>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 
 static char const* version = "1.1";
 static char const* whoami = nullptr;
@@ -16,8 +16,7 @@ static char const* whoami = nullptr;
 void
 usage()
 {
-    std::cerr << "Usage: " << whoami
-              << " --in in_file [--out out_file] [--key key [--val val]?]+\n"
+    std::cerr << "Usage: " << whoami << " --in in_file [--out out_file] [--key key [--val val]?]+\n"
               << "Modifies/Adds/Removes PDF /Info entries in the in_file\n"
               << "and stores the result in out_file\n"
               << "Special mode: " << whoami << " --dump file\n"
@@ -26,8 +25,7 @@ usage()
 }
 
 void
-dumpInfoDict(
-    QPDF& pdf, std::ostream& os = std::cout, std::string const& sep = ":\t")
+dumpInfoDict(QPDF& pdf, std::ostream& os = std::cout, std::string const& sep = ":\t")
 {
     QPDFObjectHandle trailer = pdf.getTrailer();
     if (trailer.hasKey("/Info")) {
@@ -41,7 +39,7 @@ dumpInfoDict(
             {
                 val = it.second.unparseResolved();
             }
-            os << it.first.substr(1) << sep << val << std::endl; // skip '/'
+            os << it.first.substr(1) << sep << val << '\n'; // skip '/'
         }
     }
 }
@@ -54,7 +52,7 @@ pdfDumpInfoDict(char const* fname)
         pdf.processFile(fname);
         dumpInfoDict(pdf);
     } catch (std::exception& e) {
-        std::cerr << e.what() << std::endl;
+        std::cerr << e.what() << '\n';
         exit(2);
     }
 }
@@ -68,7 +66,7 @@ main(int argc, char* argv[])
     whoami = QUtil::getWhoami(argv[0]);
 
     if ((argc == 2) && (!strcmp(argv[1], "--version"))) {
-        std::cout << whoami << " version " << version << std::endl;
+        std::cout << whoami << " version " << version << '\n';
         exit(0);
     }
     if ((argc == 3) && (!strcmp(argv[1], "--dump"))) {
@@ -92,11 +90,11 @@ main(int argc, char* argv[])
         } else if ((!strcmp(argv[i], "--key")) && (++i < argc)) {
             QTC::TC("examples", "pdf-mod-info -key");
             cur_key = argv[i];
-            if (!((cur_key.length() > 0) && (cur_key.at(0) == '/'))) {
+            if (cur_key.empty() || cur_key.at(0) != '/') {
                 cur_key = "/" + cur_key;
             }
             Keys[cur_key] = "";
-        } else if ((!strcmp(argv[i], "--val")) && (++i < argc)) {
+        } else if (!strcmp(argv[i], "--val") && ++i < argc) {
             if (cur_key.empty()) {
                 QTC::TC("examples", "pdf-mod-info usage wrong val");
                 usage();
@@ -117,7 +115,7 @@ main(int argc, char* argv[])
         QTC::TC("examples", "pdf-mod-info in-place");
         fl_out = fl_in;
     }
-    if (Keys.size() == 0) {
+    if (Keys.empty()) {
         QTC::TC("examples", "pdf-mod-info no keys");
         usage();
     }
@@ -133,7 +131,7 @@ main(int argc, char* argv[])
         QPDFObjectHandle fileinfo;
 
         for (auto const& it: Keys) {
-            if (!fileinfo.isInitialized()) {
+            if (!fileinfo) {
                 if (filetrailer.hasKey("/Info")) {
                     QTC::TC("examples", "pdf-mod-info has info");
                     fileinfo = filetrailer.getKey("/Info");
@@ -143,7 +141,7 @@ main(int argc, char* argv[])
                     filetrailer.replaceKey("/Info", fileinfo);
                 }
             }
-            if (it.second == "") {
+            if (it.second.empty()) {
                 fileinfo.removeKey(it.first);
             } else {
                 QPDFObjectHandle elt = fileinfo.newString(it.second);
@@ -157,17 +155,16 @@ main(int argc, char* argv[])
         w.setStaticID(static_id); // for testing only
         w.write();
     } catch (std::exception& e) {
-        std::cerr << e.what() << std::endl;
+        std::cerr << e.what() << '\n';
         exit(2);
     }
 
     try {
         (void)remove(fl_out);
         QUtil::os_wrapper(
-            "rename " + fl_tmp + " " + std::string(fl_out),
-            rename(fl_tmp.c_str(), fl_out));
+            "rename " + fl_tmp + " " + std::string(fl_out), rename(fl_tmp.c_str(), fl_out));
     } catch (std::exception& e) {
-        std::cerr << e.what() << std::endl;
+        std::cerr << e.what() << '\n';
         exit(2);
     }
 
