@@ -12,7 +12,7 @@ main(int argc, char** argv)
         size_t size = 0;
         QUtil::read_file_into_memory(argv[i], file_buf, size);
         LLVMFuzzerTestOneInput(reinterpret_cast<unsigned char*>(file_buf.get()), size);
-        std::cout << argv[i] << " successful" << std::endl;
+        std::cout << argv[i] << " successful" << '\n';
     }
     return 0;
 }
