@@ -2,12 +2,7 @@
 
 #include <qpdf/QUtil.hh>
 #include <qpdf/qpdf-config.h>
-#include <stdlib.h>
-
-InsecureRandomDataProvider::InsecureRandomDataProvider() :
-    seeded_random(false)
-{
-}
+#include <cstdlib>
 
 void
 InsecureRandomDataProvider::provideRandomData(unsigned char* data, size_t len)
@@ -21,11 +16,9 @@ long
 InsecureRandomDataProvider::random()
 {
     if (!this->seeded_random) {
-        // Seed the random number generator with something simple, but
-        // just to be interesting, don't use the unmodified current
-        // time.  It would be better if this were a more secure seed.
-        unsigned int seed =
-            static_cast<unsigned int>(QUtil::get_current_time() ^ 0xcccc);
+        // Seed the random number generator with something simple, but just to be interesting, don't
+        // use the unmodified current time.  It would be better if this were a more secure seed.
+        auto seed = static_cast<unsigned int>(QUtil::get_current_time() ^ 0xcccc);
 #ifdef HAVE_RANDOM
         ::srandom(seed);
 #else
