@@ -2,16 +2,15 @@
 #include <qpdf/Pl_StdioFile.hh>
 #include <qpdf/QUtil.hh>
 
+#include <cstdio>
+#include <cstdlib>
 #include <iostream>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 
 int
 main(int argc, char* argv[])
 {
     if (argc != 3) {
-        std::cerr << "Usage: dct_uncompress infile outfile" << std::endl;
+        std::cerr << "Usage: dct_uncompress infile outfile" << '\n';
         exit(2);
     }
 
