@@ -5,10 +5,10 @@
 #include <qpdf/QIntC.hh>
 #include <qpdf/QUtil.hh>
 
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <iostream>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 
 static void
 other_tests()
@@ -20,7 +20,7 @@ other_tests()
     memcpy(data.get(), "potato", 6);
     r.process(data.get(), 6, data.get());
     assert(memcmp(data.get(), "\xa5\x6f\xe7\x27\x2b\x5c", 6) == 0);
-    std::cout << "passed" << std::endl;
+    std::cout << "passed" << '\n';
 }
 
 int
@@ -32,7 +32,7 @@ main(int argc, char* argv[])
     }
 
     if (argc != 4) {
-        std::cerr << "Usage: rc4 hex-key infile outfile" << std::endl;
+        std::cerr << "Usage: rc4 hex-key infile outfile" << '\n';
         exit(2);
     }
 
@@ -41,7 +41,7 @@ main(int argc, char* argv[])
     char* outfilename = argv[3];
     unsigned int hexkeylen = QIntC::to_uint(strlen(hexkey));
     unsigned int keylen = hexkeylen / 2;
-    unsigned char* key = new unsigned char[keylen + 1];
+    auto* key = new unsigned char[keylen + 1];
     key[keylen] = '\0';
 
     FILE* infile = QUtil::safe_fopen(infilename, "rb");
@@ -51,14 +51,15 @@ main(int argc, char* argv[])
         t[1] = hexkey[i + 1];
         t[2] = '\0';
 
-        long val = strtol(t, 0, 16);
+        long val = strtol(t, nullptr, 16);
         key[i / 2] = static_cast<unsigned char>(val);
     }
 
     FILE* outfile = QUtil::safe_fopen(outfilename, "wb");
-    Pl_StdioFile* out = new Pl_StdioFile("stdout", outfile);
+    auto* out = new Pl_StdioFile("stdout", outfile);
     // Use a small buffer size (64) for testing
-    Pl_RC4* rc4 = new Pl_RC4("rc4", out, key, QIntC::to_int(keylen), 64U);
+    std::string keystr(reinterpret_cast<char const*>(key), keylen);
+    auto* rc4 = new Pl_RC4("rc4", out, keystr, 64U);
     delete[] key;
 
     // 64 < buffer size < 512, buffer_size is not a power of 2 for testing
